/*
 * Copyright (c) 2005 Eugene Kuleshov
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.javatx.spring.aop;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.jmock.Mock;
import org.jmock.core.Stub;
import org.jmock.core.constraint.IsEqual;
import org.jmock.core.matcher.InvokeOnceMatcher;
import org.jmock.core.stub.ReturnStub;
import org.jmock.core.stub.ThrowStub;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;


public class ForeignTradeAdviceTest extends TestCase {
  private final String CTX_NAME = "/"+getClass().getPackage().getName().replace('.', '/')+"/applicationContext.xml";

  private TradeManager tradeManager;
  private Mock baseTradeManagerMock;
  private Mock yahooTradeManagerMock;

  private final boolean baseCall;
  private final Stub baseWill;
  private final boolean yahooCall;
  private final Stub yahooWill;
  private final BigDecimal expectedValue;
  private final Throwable expectedException;

  public static TestSuite suite() {
    BigDecimal v1 = new BigDecimal( "0.22");
    BigDecimal v2 = new BigDecimal( "0.33");
    
    RuntimeException e1 = new RuntimeException( "e1");
    RuntimeException e2 = new RuntimeException( "e2");
    
    TestSuite suite = new TestSuite(ForeignTradeAdviceTest.class.getName());
    suite.addTest( new ForeignTradeAdviceTest(true, v1, null,   false, null, null, v1,   null));
    suite.addTest( new ForeignTradeAdviceTest(true, null, e1,   false, null, null, null, e1));
    suite.addTest( new ForeignTradeAdviceTest(true, null, null, true,  v2,   null, v2,   null));
    suite.addTest( new ForeignTradeAdviceTest(true, null, null, true,  null, null, null, null));
    suite.addTest( new ForeignTradeAdviceTest(true, null, null, true,  null, e2,   null, e2));
    return suite;
  }
  
  public ForeignTradeAdviceTest(
      boolean baseCall, BigDecimal baseValue, Throwable baseException,
      boolean yahooCall, BigDecimal yahooValue, Throwable yahooException,
      BigDecimal expectedValue, Throwable expectedException) {
    super("test");
  
    this.baseCall = baseCall;
    this.baseWill = baseException==null ? 
        (Stub) new ReturnStub(baseValue) : new ThrowStub(baseException);
    this.yahooCall = yahooCall;
    this.yahooWill = yahooException==null ? 
        (Stub) new ReturnStub(yahooValue) : new ThrowStub(yahooException);
    this.expectedValue = expectedValue;
    this.expectedException = expectedException;
  }
  
  protected void setUp() throws Exception {
    super.setUp();

    baseTradeManagerMock = new Mock(TradeManager.class, "baseTradeManager");
    TradeManager baseTradeManager = ( TradeManager) baseTradeManagerMock.proxy();
    
    yahooTradeManagerMock = new Mock(TradeManager.class, "yahooTradeManager");
    TradeManager yahooTradeManager = ( TradeManager) yahooTradeManagerMock.proxy();

    Map stubs = new HashMap();
    stubs.put("yahooFeed", yahooTradeManager);
    stubs.put("baseTradeManager", baseTradeManager);
    
    ConfigurableApplicationContext ctx = new ClassPathXmlApplicationContext(CTX_NAME);
    ctx.getBeanFactory().addBeanPostProcessor(new StubPostProcessor(stubs));
    
    tradeManager = ( TradeManager) ctx.getBean("tradeManager");
  }
  
  public void test() throws Throwable {
    String symbol = "testSymbol";

    if(baseCall) {
      baseTradeManagerMock.expects(new InvokeOnceMatcher())
        .method("getPrice").with(new IsEqual(symbol)).will(baseWill);
    }
    
    if(yahooCall) {
      yahooTradeManagerMock.expects(new InvokeOnceMatcher())
        .method("getPrice").with(new IsEqual(symbol)).will(yahooWill);
    }
    
    try {
      BigDecimal price = tradeManager.getPrice(symbol);
      assertEquals("Invalid price", expectedValue, price);
    } catch(Exception e) {
      if(expectedException==null) {
        throw e;
      }
    }
    
    baseTradeManagerMock.verify();
    yahooTradeManagerMock.verify();
  }
  
  public String getName() {
    StringBuffer sb = new StringBuffer(super.getName());
    sb.append(" ").append(baseCall).append(" ");
    baseWill.describeTo(sb);
    sb.append(" ").append(yahooCall).append(" ");
    yahooWill.describeTo(sb);
    sb.append(" ").append(expectedValue);
    sb.append(" ").append(expectedException);
    return sb.toString();
  }
  
}

