/*
 * Copyright (c) 2005 Eugene Kuleshov
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.javatx.spring.aop;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import junit.framework.TestCase;

import org.jmock.Mock;
import org.jmock.core.Invocation;
import org.jmock.core.constraint.IsEqual;
import org.jmock.core.matcher.InvokeOnceMatcher;
import org.jmock.core.stub.CustomStub;
import org.springframework.aop.framework.ProxyFactory;
import org.springframework.aop.support.NameMatchMethodPointcutAdvisor;


public class SyncPointAdviceTest extends TestCase {
  private static final int THREAD_COUNT = 20;
  private static final String SYMBOL = "something";
  
  Mock tradeManagerMock;
  TradeManager tradeManager;

  Mock resultsMock;
  Map results;

  Set threads;
  

  protected void setUp() throws Exception {
    tradeManagerMock = new Mock(TradeManager.class, "tradeManager");
    NameMatchMethodPointcutAdvisor syncPointAdvisor = new NameMatchMethodPointcutAdvisor();
    syncPointAdvisor.setAdvice(new SyncPointAdvice());
    syncPointAdvisor.setMappedName("getPrice");
    
    ProxyFactory proxyFactory = new ProxyFactory(tradeManagerMock.proxy());
    proxyFactory.addAdvisor(syncPointAdvisor);
    tradeManager = (TradeManager) proxyFactory.getProxy();

    resultsMock = new Mock(Map.class);
    results = (Map) resultsMock.proxy();
    
    threads = Collections.synchronizedSet(new HashSet());
  }
  
  public void testSyncPoint() throws Exception {
    final BigDecimal expectedPrice = new BigDecimal(0.123);
    final long timeout = 100L;

    tradeManagerMock.expects(new InvokeOnceMatcher()).method("getPrice")
      .withAnyArguments().will( new CustomStub("req") {
          public Object invoke( Invocation i) throws Throwable {
            Thread.sleep(timeout);
            return expectedPrice;
          }
        });
    
    for( int i = 0; i<THREAD_COUNT; i++) {
      Thread t = new WorkerThread();
      resultsMock.expects(new InvokeOnceMatcher()).method("put")
        .with(new IsEqual(t), new IsEqual(expectedPrice));
      threads.add(t);
    }  
      
    Thread[] tt = (Thread[]) this.threads.toArray(new Thread[0]);
    for( int i = 0; i < tt.length; i++) {
      tt[i].start();
    }
    
    while(this.threads.size()>0) {
      synchronized(this.threads) {
        this.threads.wait(timeout);
      }
    }
    
    tradeManagerMock.verify();
    resultsMock.verify();
  }

  
  private final class WorkerThread extends Thread {
    public void run() {
      Thread thread = Thread.currentThread();
      try {
        results.put(thread, tradeManager.getPrice(SYMBOL));
      } finally {
        synchronized(threads) {
          threads.remove(thread);
          threads.notifyAll();
        }
      }
    }
  }
  
}

